==> VLC-Android-sources-2.9.1.tar.gz: vlc-android/src/org/videolan/vlc/gui/DiffUtilAdapter.kt <==
package org.videolan.vlc.gui

import android.support.annotation.WorkerThread
import android.support.v7.util.DiffUtil
import android.support.v7.widget.RecyclerView
import android.util.Log
import kotlinx.coroutines.experimental.android.UI
import kotlinx.coroutines.experimental.channels.Channel
import kotlinx.coroutines.experimental.channels.actor
import kotlinx.coroutines.experimental.launch
import kotlinx.coroutines.experimental.newSingleThreadContext
import java.util.*

abstract class DiffUtilAdapter<D, VH : RecyclerView.ViewHolder> : RecyclerView.Adapter<VH>() {

    protected var dataset: List<D> = listOf()
    @Volatile private var last = dataset
    private val diffCallback by lazy(LazyThreadSafetyMode.NONE) { createCB() }
    private val updateActor = actor<List<D>>(newSingleThreadContext("vlc-updater"), capacity = Channel.CONFLATED) {
        for (list in channel) internalUpdate(list)
    }
    protected abstract fun onUpdateFinished()

    fun update (list: List<D>) {
        last = list
        updateActor.offer(list)
    }

    @WorkerThread
    private suspend fun internalUpdate(list: List<D>) {
        Log.d("dua", "old list ${dataset.size} -> ${list.size}")
        val finalList = prepareList(list)
        val result = DiffUtil.calculateDiff(diffCallback.apply { update(dataset, finalList) }, detectMoves())
        launch(UI) {
            dataset = finalList
            result.dispatchUpdatesTo(this@DiffUtilAdapter)
            onUpdateFinished()
        }.join()
    }

    open protected fun prepareList(list: List<D>) : List<D> = ArrayList(list)

    fun peekLast() = last

    fun hasPendingUpdates() = updateActor.isFull

    open protected fun detectMoves() = false

    open protected fun createCB() = DiffCallback<D>()

    open class DiffCallback<D> : DiffUtil.Callback() {
        lateinit var oldList: List<D>
        lateinit var newList: List<D>

        fun update(oldList: List<D>, newList: List<D>) {
            this.oldList = oldList
            this.newList = newList
        }

        override fun getOldListSize() = oldList.size

        override fun getNewListSize() = newList.size

        override fun areContentsTheSame(oldItemPosition : Int, newItemPosition : Int) = true

        override fun areItemsTheSame(oldItemPosition : Int, newItemPosition : Int) = oldList[oldItemPosition] == newList[newItemPosition]
    }
}

==> VLC-Android-sources-3.0.0.tar.gz: vlc-android/src/org/videolan/vlc/gui/DiffUtilAdapter.kt <==
package org.videolan.vlc.gui

import android.support.annotation.MainThread
import android.support.annotation.WorkerThread
import android.support.v7.util.DiffUtil
import android.support.v7.widget.RecyclerView
import kotlinx.coroutines.experimental.android.UI
import kotlinx.coroutines.experimental.channels.Channel
import kotlinx.coroutines.experimental.channels.actor
import kotlinx.coroutines.experimental.launch
import kotlinx.coroutines.experimental.newSingleThreadContext
import java.util.*

abstract class DiffUtilAdapter<D, VH : RecyclerView.ViewHolder> : RecyclerView.Adapter<VH>() {

    protected var dataset: List<D> = listOf()
    private set
    @Volatile private var last = dataset
    private val diffCallback by lazy(LazyThreadSafetyMode.NONE) { createCB() }
    private val updateActor = actor<List<D>>(newSingleThreadContext("vlc-updater"), capacity = Channel.CONFLATED) {
        for (list in channel) internalUpdate(list)
    }
    protected abstract fun onUpdateFinished()

    @MainThread
    fun update (list: List<D>) {
        last = list
        updateActor.offer(list)
    }

    @WorkerThread
    private suspend fun internalUpdate(list: List<D>) {
        val finalList = prepareList(list)
        val result = DiffUtil.calculateDiff(diffCallback.apply { update(dataset, finalList) }, detectMoves())
        launch(UI) {
            dataset = finalList
            result.dispatchUpdatesTo(this@DiffUtilAdapter)
            onUpdateFinished()
        }.join()
    }

    protected open fun prepareList(list: List<D>) : List<D> = ArrayList(list)

    fun peekLast() = last

    fun hasPendingUpdates() = updateActor.isFull

    protected open fun detectMoves() = false

    protected open fun createCB() = DiffCallback<D>()

    open class DiffCallback<D> : DiffUtil.Callback() {
        lateinit var oldList: List<D>
        lateinit var newList: List<D>

        fun update(oldList: List<D>, newList: List<D>) {
            this.oldList = oldList
            this.newList = newList
        }

        override fun getOldListSize() = oldList.size

        override fun getNewListSize() = newList.size

        override fun areContentsTheSame(oldItemPosition : Int, newItemPosition : Int) = true

        override fun areItemsTheSame(oldItemPosition : Int, newItemPosition : Int) = oldList[oldItemPosition] == newList[newItemPosition]
    }
}
